@@ -41,9 +41,13 @@
 //! ```
 //!
 use std::fmt::Debug;
+use std::str::FromStr;
 
+use regex::Regex;
 use serde::de::DeserializeOwned;
 
+use asserter::body::{assert_body_regex, assert_bytes_body, assert_json_body_eq, assert_text_body};
+
 #[cfg(feature = "surf")]
 mod assert_surf;
 #[cfg(feature = "isahc")]
@@ -516,6 +520,38 @@ pub trait AsserhttpHeader<T> {
 
 /// For assertions on http response body
 pub trait AsserhttpBody<T> {
+    /// Allows verifying json body in a closure
+    /// * `asserter` - closure to verify json body
+    ///
+    /// # Example
+    /// ```no_run
+    /// # use isahc;
+    /// # use surf;
+    /// # use serde::Deserialize;
+    /// use asserhttp::*;
+    /// use serde_json::{json, Value};
+    ///
+    /// #[derive(Deserialize, Debug, Eq, PartialEq)]
+    /// struct MyStruct { a: String }
+    ///
+    /// #[async_std::main]
+    /// async fn main() {
+    ///     isahc::get("http://localhost").unwrap().expect_body_json(|b: Value| assert_eq!(b, json!({"a": "b"})));
+    ///     isahc::get("http://localhost").expect_body_json(|b: Value| assert_eq!(b, json!({"a": "b"})));
+    ///     isahc::get_async("http://localhost").await.unwrap().expect_body_json(|b: Value| assert_eq!(b, json!({"a": "b"})));
+    ///     isahc::get_async("http://localhost").await.expect_body_json(|b: Value| assert_eq!(b, json!({"a": "b"})));
+    ///
+    ///     surf::get("http://localhost").await.unwrap().expect_body_json(|b: Value| assert_eq!(b, json!({"a": "b"})));
+    ///     surf::get("http://localhost").await.expect_body_json(|b: Value| assert_eq!(b, json!({"a": "b"})));
+    ///
+    ///     // or use your structs
+    ///     isahc::get("http://localhost").expect_body_json(|b: MyStruct| assert_eq!(b, MyStruct { a: String::from("b") }));
+    /// }
+    /// ```
+    fn expect_body_json<B, F>(&mut self, asserter: F) -> &mut T
+        where B: DeserializeOwned + PartialEq + Debug + Unpin,
+              F: FnOnce(B);
+
     /// Expects response body to be json and equal
     /// * `body` - expected json body
     ///
@@ -528,14 +564,147 @@ pub trait AsserhttpBody<T> {
     ///
     /// #[async_std::main]
     /// async fn main() {
-    ///     isahc::get("http://localhost").unwrap().expect_body_json(json!({"a": "b"}));
-    ///     isahc::get("http://localhost").expect_body_json(json!({"a": "b"}));
-    ///     isahc::get_async("http://localhost").await.unwrap().expect_body_json(json!({"a": "b"}));
-    ///     isahc::get_async("http://localhost").await.expect_body_json(json!({"a": "b"}));
+    ///     isahc::get("http://localhost").unwrap().expect_body_json_eq(json!({"a": "b"}));
+    ///     isahc::get("http://localhost").expect_body_json_eq(json!({"a": "b"}));
+    ///     isahc::get_async("http://localhost").await.unwrap().expect_body_json_eq(json!({"a": "b"}));
+    ///     isahc::get_async("http://localhost").await.expect_body_json_eq(json!({"a": "b"}));
     ///
-    ///     surf::get("http://localhost").await.unwrap().expect_body_json(json!({"a": "b"}));
-    ///     surf::get("http://localhost").await.expect_body_json(json!({"a": "b"}));
+    ///     surf::get("http://localhost").await.unwrap().expect_body_json_eq(json!({"a": "b"}));
+    ///     surf::get("http://localhost").await.expect_body_json_eq(json!({"a": "b"}));
     /// }
     /// ```
-    fn expect_body_json<B>(&mut self, body: B) -> &mut T where B: DeserializeOwned + PartialEq + Debug + Unpin;
+    fn expect_body_json_eq<B>(&mut self, body: B) -> &mut T where B: DeserializeOwned + PartialEq + Debug + Unpin {
+        self.expect_body_json(|actual: B| assert_json_body_eq(actual, body))
+    }
+
+    /// Allows verifying text body in a closure
+    /// * `asserter` - closure to verify text body
+    ///
+    /// # Example
+    /// ```no_run
+    /// # use isahc;
+    /// # use surf;
+    /// use asserhttp::*;
+    ///
+    /// #[async_std::main]
+    /// async fn main() {
+    ///     isahc::get("http://localhost").unwrap().expect_body_text(|b| assert_eq!(b, String::from("abcd")));
+    ///     isahc::get("http://localhost").unwrap().expect_body_text(|b| assert_eq!(b, "abcd"));
+    ///     isahc::get("http://localhost").expect_body_text(|b| assert_eq!(b, "abcd"));
+    ///     isahc::get_async("http://localhost").await.unwrap().expect_body_text(|b| assert_eq!(b, "abcd"));
+    ///     isahc::get_async("http://localhost").await.expect_body_text(|b| assert_eq!(b, "abcd"));
+    ///
+    ///     surf::get("http://localhost").await.unwrap().expect_body_text(|b| assert_eq!(b, "abcd"));
+    ///     surf::get("http://localhost").await.expect_body_text(|b| assert_eq!(b, "abcd"));
+    /// }
+    /// ```
+    fn expect_body_text<F>(&mut self, asserter: F) -> &mut T where F: FnOnce(String);
+
+    /// Expects response body to be text and equal
+    /// * `body` - expected text body
+    ///
+    /// # Example
+    /// ```no_run
+    /// # use isahc;
+    /// # use surf;
+    /// use asserhttp::*;
+    ///
+    /// #[async_std::main]
+    /// async fn main() {
+    ///     isahc::get("http://localhost").unwrap().expect_body_text_eq(String::from("abcd"));
+    ///     isahc::get("http://localhost").unwrap().expect_body_text_eq("abcd");
+    ///     isahc::get("http://localhost").expect_body_text_eq("abcd");
+    ///     isahc::get_async("http://localhost").await.unwrap().expect_body_text_eq("abcd");
+    ///     isahc::get_async("http://localhost").await.expect_body_text_eq("abcd");
+    ///
+    ///     surf::get("http://localhost").await.unwrap().expect_body_text_eq("abcd");
+    ///     surf::get("http://localhost").await.expect_body_text_eq("abcd");
+    /// }
+    /// ```
+    fn expect_body_text_eq<B>(&mut self, body: B) -> &mut T where B: Into<String> {
+        self.expect_body_text(|actual| assert_text_body(actual, body.into()))
+    }
+
+    /// Expects response body to be text and to match provided regex
+    /// * `regex` - must match text response body
+    ///
+    /// # Example
+    /// ```no_run
+    /// # use isahc;
+    /// # use surf;
+    /// use asserhttp::*;
+    ///
+    /// #[async_std::main]
+    /// async fn main() {
+    ///     isahc::get("http://localhost").unwrap().expect_body_text_matches(String::from("[a-z]+"));
+    ///     isahc::get("http://localhost").unwrap().expect_body_text_matches("[a-z]+");
+    ///     isahc::get("http://localhost").expect_body_text_matches("[a-z]+");
+    ///     isahc::get_async("http://localhost").await.unwrap().expect_body_text_matches("[a-z]+");
+    ///     isahc::get_async("http://localhost").await.expect_body_text_matches("[a-z]+");
+    ///
+    ///     surf::get("http://localhost").await.unwrap().expect_body_text_matches("[a-z]+");
+    ///     surf::get("http://localhost").await.expect_body_text_matches("[a-z]+");
+    /// }
+    /// ```
+    fn expect_body_text_matches<R>(&mut self, regex: R) -> &mut T where R: Into<String> {
+        let regex = Regex::from_str(regex.into().as_str())
+            .expect("'{}' is not a valid regex");
+        self.expect_body_text(|actual| assert_body_regex(actual, regex))
+    }
+
+    /// Allows verifying response body bytes in a closure
+    /// * `asserter` - closure to verify response body as bytes
+    ///
+    /// # Example
+    /// ```no_run
+    /// # use isahc;
+    /// # use surf;
+    /// use asserhttp::*;
+    ///
+    /// #[async_std::main]
+    /// async fn main() {
+    ///     isahc::get("http://localhost").expect_body_bytes(|b| assert_eq!(b, b"abcd"));
+    ///     isahc::get("http://localhost").expect_body_bytes(|b| assert_eq!(b, &[97, 98, 99, 100]));
+    ///     isahc::get("http://localhost").expect_body_bytes(|b| assert_eq!(b, &vec![97, 98, 99, 100]));
+    ///
+    ///     // on any client
+    ///     isahc::get("http://localhost").unwrap().expect_body_bytes(|b| assert_eq!(b, b"abcd"));
+    ///     isahc::get("http://localhost").expect_body_bytes(|b| assert_eq!(b, b"abcd"));
+    ///     isahc::get_async("http://localhost").await.unwrap().expect_body_bytes(|b| assert_eq!(b, b"abcd"));
+    ///     isahc::get_async("http://localhost").await.expect_body_bytes(|b| assert_eq!(b, b"abcd"));
+    ///
+    ///     surf::get("http://localhost").await.unwrap().expect_body_bytes(|b| assert_eq!(b, b"abcd"));
+    ///     surf::get("http://localhost").await.expect_body_bytes(|b| assert_eq!(b, b"abcd"));
+    /// }
+    /// ```
+    fn expect_body_bytes<F>(&mut self, asserter: F) -> &mut T where F: FnOnce(&[u8]);
+
+    /// Expects response body to be equal by comparing bytes
+    /// * `body` - expected bytes response body
+    ///
+    /// # Example
+    /// ```no_run
+    /// # use isahc;
+    /// # use surf;
+    /// use asserhttp::*;
+    ///
+    /// #[async_std::main]
+    /// async fn main() {
+    ///     isahc::get("http://localhost").expect_body_bytes_eq(b"abcd");
+    ///     isahc::get("http://localhost").expect_body_bytes_eq(&[97, 98, 99, 100]);
+    ///     isahc::get("http://localhost").expect_body_bytes_eq(&vec![97, 98, 99, 100]);
+    ///
+    ///     // on any client
+    ///     isahc::get("http://localhost").unwrap().expect_body_bytes_eq(b"abcd");
+    ///     isahc::get("http://localhost").expect_body_bytes_eq(b"abcd");
+    ///     isahc::get_async("http://localhost").await.unwrap().expect_body_bytes_eq(b"abcd");
+    ///     isahc::get_async("http://localhost").await.expect_body_bytes_eq(b"abcd");
+    ///
+    ///     surf::get("http://localhost").await.unwrap().expect_body_bytes_eq(b"abcd");
+    ///     surf::get("http://localhost").await.expect_body_bytes_eq(b"abcd");
+    /// }
+    /// ```
+    fn expect_body_bytes_eq(&mut self, body: &[u8]) -> &mut T {
+        self.expect_body_bytes(|actual| assert_bytes_body(actual, body))
+    }
 }
\ No newline at end of file
